#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#define M 20
#define N 10
#define NO_DOORS 200

int visited[M][N];
int H[M][N+1], V[M+1][N];

typedef struct _node {
   int x, y;
   struct _node *next;
} node;

node *init ()
{
   node *p;

   p = (node *)malloc(sizeof(node));
   p -> next = NULL;
   p -> x = p -> y = -1;
   return p;
}

int isempty ( node *S )
{
   return (S -> next == NULL);
}

node *push ( node *S, int i, int j )
{
   node *p;

   p = (node *)malloc(sizeof(node));
   p -> x = i;
   p -> y = j;
   p -> next = S -> next;
   S -> next = p;
   return S;
}

node *pop ( node *S )
{
   node *p;

   p = S -> next;
   if (p != NULL) {
      S -> next = S -> next -> next;
      free(p);
   }
   return S;
}

int createDoor ( int flag )
{
   int i, j;

   i = 1 + rand() % (M - 1);
   j = 1 + rand() % (N - 1);
   if (flag == 0) {
      if (H[i][j]) return 0;
      H[i][j] = 1;
   } else {
      if (V[i][j]) return 0;
      V[i][j] = 1;
   }
   return 1;
}

void createAllDoors ( int nd )
{
   int i, j, k;

   for (i=0; i<M; ++i) for (j=0; j<=N; ++j) H[i][j] = 0;
   for (i=0; i<=M; ++i) for (j=0; j<N; ++j) V[i][j] = 0;
   for (i=0; i<M; ++i) for (j=0; j<N; ++j) visited[i][j] = 0;
   k = 0;
   while (k < nd) k += createDoor(rand()%2);
}

int DFS ( node *S, int u, int v )
{
   int i, j;

   while (!isempty(S)) {
      i = S -> next -> x;
      j = S -> next -> y;
      S = pop(S);
      if ((i == u) && (j == v)) return 1;
      if ((H[i][j]) && (!visited[i][j-1])) { S = push(S,i,j-1); visited[i][j-1] = 1; }
      if ((H[i][j+1]) && (!visited[i][j+1])) { S = push(S,i,j+1); visited[i][j+1] = 1; }
      if ((V[i][j]) && (!visited[i-1][j])) { S = push(S,i-1,j); visited[i-1][j] = 1; }
      if ((V[i+1][j]) && (!visited[i+1][j])) { S = push(S,i+1,j); visited[i+1][j] = 1; }
   }
   return 0;
}

void printRooms ( int m, int n )
{
   int i, j;

   for (j=n-1; j>=0; --j) {
      for (i=0; i<m; ++i) printf("+%s", H[i][j+1] ? "-   -" : "-----"); printf("+\n");
      printf("|"); for (i=0; i<m; ++i) printf("%2d,%-2d%c", i, j, V[i+1][j] ? ' ' : '|'); printf("\n");
   }
   for (i=0; i<m; ++i) printf("+%s", H[i][0] ? "-   -" : "-----"); printf("+\n");
}

int main ()
{
   node *S;
   int s, t, u, v, status;

   srand((unsigned int)time(NULL));
   createAllDoors(NO_DOORS);
   printRooms(M,N);
   S = init();
   s = rand() % M; t = rand() % N;
   u = rand() % M; v = rand() % N;
   printf("(s,t) = (%d,%d), (u,v) = (%d,%d)\n", s,t,u,v);
   S = push(S,s,t);
   visited[s][t] = 1;
   status = DFS(S,u,v);
   printf("Status returned by DFS: %d\n", status);
   exit(0);
}
